/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.info.nachricht;

import java.util.Set;

import jakarta.validation.ConstraintViolation;
import jakarta.validation.ConstraintViolationException;
import jakarta.validation.Validator;

import de.ozgcloud.info.common.InvalidNachrichtenListUrlException;
import de.ozgcloud.info.common.NachrichtEvent;
import io.grpc.Status;
import io.grpc.stub.StreamObserver;
import lombok.RequiredArgsConstructor;
import net.devh.boot.grpc.server.service.GrpcService;

@GrpcService
@RequiredArgsConstructor
public class NachrichtGrpcService extends NachrichtServiceGrpc.NachrichtServiceImplBase {
	static final String STATUS_OK = "ok";

	private final NachrichtService nachrichtService;
	private final NachrichtMapper nachrichtenMapper;
	private final Validator validator;

	@Override
	public void saveNewNachricht(GrpcNewNachrichtRequest request, StreamObserver<GrpcNewNachrichtReply> responseObserver) {
		try {
			saveNachricht(request);
		} catch (InvalidNachrichtenListUrlException | ConstraintViolationException e) {
			responseObserver.onError(Status.INVALID_ARGUMENT.withDescription(e.getMessage()).asRuntimeException());
		}

		responseObserver.onNext(GrpcNewNachrichtReply.newBuilder().setStatus(STATUS_OK).build());
		responseObserver.onCompleted();
	}

	private void saveNachricht(GrpcNewNachrichtRequest request) {
		var nachricht = nachrichtenMapper.fromGrpcNachricht(request.getNachricht());

		checkForValidity(nachricht);
		
		nachrichtService.save(nachricht);
	}

	private void checkForValidity(NachrichtEvent nachrichtEvent) {
		Set<ConstraintViolation<NachrichtEvent>> violations = validator.validate(nachrichtEvent);
		if (!violations.isEmpty()) {
			throw new ConstraintViolationException(violations);
		}
	}

	@Override
	public void finishNachricht(GrpcFinishedNachrichtRequest request, StreamObserver<GrpcFinishedNachrichtReply> responseObserver) {
		nachrichtService.finish(nachrichtenMapper.fromGrpcNachricht(request.getNachricht()));

		responseObserver.onNext(GrpcFinishedNachrichtReply.newBuilder().setStatus(STATUS_OK).build());
		responseObserver.onCompleted();
	}
}
