/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.info.information;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.NoSuchElementException;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;

import de.ozgcloud.info.GrpcInformationByIdRequest;
import de.ozgcloud.info.GrpcInformationByIdResponse;
import de.ozgcloud.info.GrpcInformationNachricht;
import de.ozgcloud.info.GrpcInformationRequest;
import de.ozgcloud.info.GrpcInformationResponse;
import de.ozgcloud.info.NachrichtEventTestFactory;
import de.ozgcloud.info.common.NachrichtEvent;
import io.grpc.stub.StreamObserver;

@ExtendWith(MockitoExtension.class)
class InformationGrpcServiceTest {
	@Spy
	@InjectMocks
	InformationGrpcService informationGrpcService;

	@Mock
	InformationService service;

	@Spy
	StreamObserver<GrpcInformationResponse> informationResponseStreamObserver;

	@Spy
	StreamObserver<GrpcInformationByIdResponse> informationByIdResponseStreamObserver;

	@Captor
	ArgumentCaptor<GrpcInformationResponse> informationResponseArgumentCaptor;

	@Captor
	ArgumentCaptor<GrpcInformationByIdResponse> informationByIdResponseArgumentCaptor;

	@Nested
	class TestInformationAvailable {
		final NachrichtEvent firstNachrichtEvent = NachrichtEventTestFactory.createNachrichtEvent();

		@BeforeEach
		void init() {
			var information = List.of(firstNachrichtEvent, NachrichtEventTestFactory.createNachrichtEvent());
			when(service.getInformationOfPostfach(anyString())).thenReturn(information);
		}

		@Test
		void shouldCallService() {
			informationGrpcService.getInformation(
			  GrpcInformationRequest.newBuilder().setPostfachId(NachrichtEventTestFactory.POSTKORB_HANDLE).build(),
			  informationResponseStreamObserver);

			verify(service, atMostOnce()).getInformationOfPostfach(anyString());
		}

		@Test
		void shouldCreateInformation() {
			informationGrpcService.getInformation(
			  GrpcInformationRequest.newBuilder().setPostfachId(NachrichtEventTestFactory.POSTKORB_HANDLE).build(),
			  informationResponseStreamObserver);

			verify(informationResponseStreamObserver, atMostOnce()).onNext(informationResponseArgumentCaptor.capture());

			var response = informationResponseArgumentCaptor.getValue();

			assertThat(response.getNachrichtenList()).hasSize(2);

			var information = response.getNachrichtenList().getFirst();

			assertThat(information.getPostfachId()).isEqualTo(firstNachrichtEvent.getPostfachId());
			assertThat(information.getOzgCloudAddress()).isEqualTo(firstNachrichtEvent.getNachrichtenListUrl());
		}

		@Test
		void shouldSetResponse() {
			informationGrpcService.getInformation(
			  GrpcInformationRequest.newBuilder().setPostfachId(NachrichtEventTestFactory.POSTKORB_HANDLE).build(),
			  informationResponseStreamObserver);

			verify(informationResponseStreamObserver, atMostOnce()).onNext(any(GrpcInformationResponse.class));
		}
	}

	@Nested
	class TestNoInformationAvailable {
		@Test
		void responseShouldHaveEmptyList() {
			when(service.getInformationOfPostfach(anyString())).thenReturn(List.of());

			informationGrpcService.getInformation(
			  GrpcInformationRequest.newBuilder().setPostfachId(NachrichtEventTestFactory.POSTKORB_HANDLE).build(),
			  informationResponseStreamObserver);

			verify(informationResponseStreamObserver, atMostOnce()).onNext(informationResponseArgumentCaptor.capture());

			var response = informationResponseArgumentCaptor.getValue();

			assertThat(response.getNachrichtenList()).isEmpty();
		}

		@Test
		void shouldSetResponse() {
			informationGrpcService.getInformation(
			  GrpcInformationRequest.newBuilder().setPostfachId(NachrichtEventTestFactory.POSTKORB_HANDLE).build(),
			  informationResponseStreamObserver);

			verify(informationResponseStreamObserver, atMostOnce()).onNext(any(GrpcInformationResponse.class));
		}
	}

	@Nested
	class TestInformationByIdAvailable {
		final NachrichtEvent nachrichtEvent = NachrichtEventTestFactory.createNachrichtEvent();

		@BeforeEach
		void init() {
			when(service.getInformationById(NachrichtEventTestFactory.ID.toHexString())).thenReturn(nachrichtEvent);
		}

		@Test
		void shouldCallService() {
			informationGrpcService.getInformationById(
			  GrpcInformationByIdRequest.newBuilder().setId(NachrichtEventTestFactory.ID.toHexString()).build(),
			  informationByIdResponseStreamObserver);

			verify(service, atMostOnce()).getInformationById(NachrichtEventTestFactory.ID.toHexString());
		}

		@Test
		void shouldReturnInformation() {
			informationGrpcService.getInformationById(
			  GrpcInformationByIdRequest.newBuilder().setId(NachrichtEventTestFactory.ID.toHexString()).build(),
			  informationByIdResponseStreamObserver);

			verify(informationByIdResponseStreamObserver, atMostOnce()).onNext(informationByIdResponseArgumentCaptor.capture());

			var response = informationByIdResponseArgumentCaptor.getValue();
			var nachricht = response.getNachricht();

			assertThat(nachricht).isInstanceOf(GrpcInformationNachricht.class);
			assertThat(nachricht.getId()).isEqualTo(nachrichtEvent.getId());
			assertThat(nachricht.getPostfachId()).isEqualTo(nachrichtEvent.getPostfachId());
			assertThat(nachricht.getOzgCloudAddress()).isEqualTo(nachrichtEvent.getNachrichtenListUrl());
		}

		@Test
		void shouldSetResponse() {
			informationGrpcService.getInformationById(
			  GrpcInformationByIdRequest.newBuilder().setId(NachrichtEventTestFactory.ID.toHexString()).build(),
			  informationByIdResponseStreamObserver);

			verify(informationByIdResponseStreamObserver, atMostOnce()).onNext(any(GrpcInformationByIdResponse.class));
		}
	}

	@Nested
	class TestNoInformationByIdAvailable {
		@Test
		void shouldCallOnError() {
			when(service.getInformationById(NachrichtEventTestFactory.ID.toHexString())).thenThrow(new NoSuchElementException("No value present"));

			informationGrpcService.getInformationById(
			  GrpcInformationByIdRequest.newBuilder().setId(NachrichtEventTestFactory.ID.toHexString()).build(),
			  informationByIdResponseStreamObserver);

			verify(informationByIdResponseStreamObserver, atMostOnce()).onError(any());
		}
	}

	@Test
	void shouldCallOnCompleteForGrpcInformationRequest() {
		informationGrpcService.getInformation(GrpcInformationRequest.newBuilder().setPostfachId(NachrichtEventTestFactory.POSTKORB_HANDLE).build(),
		  informationResponseStreamObserver);

		verify(informationResponseStreamObserver, atMostOnce()).onCompleted();
	}
}