/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.info.nachricht;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Set;

import jakarta.validation.ConstraintViolation;
import jakarta.validation.ConstraintViolationException;
import jakarta.validation.Validator;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;

import de.ozgcloud.info.NachrichtEventTestFactory;
import de.ozgcloud.info.common.InvalidNachrichtenListUrlException;
import de.ozgcloud.info.common.NachrichtEvent;
import io.grpc.stub.StreamObserver;

@ExtendWith(MockitoExtension.class)
class NachrichtenGrpcServiceTest {
	@Spy
	@InjectMocks
	private NachrichtGrpcService nachrichtenGrpcService;

	@Mock
	private NachrichtService nachrichtenService;

	@Mock
	private NachrichtMapper nachrichtenMapper;

	@Mock
	private Validator validator;

	@Captor
	private ArgumentCaptor<NachrichtEvent> nachrichtArgumentCaptor;

	@Nested
	class TestSaveNewNachrichtEvent {
		@Mock
		private StreamObserver<GrpcNewNachrichtReply> newNachrichtStreamObserver;

		private final GrpcNewNachrichtRequest newNachrichtRequest = NachrichtEventTestFactory.createNewNachrichtRequest();

		@BeforeEach
		void init() {
			when(nachrichtenMapper.fromGrpcNachricht(newNachrichtRequest.getNachricht())).thenReturn(
			  NachrichtEventTestFactory.createNachrichtEventBuilder().id(null).build());

			when(validator.validate(any(NachrichtEvent.class))).thenReturn(Set.of());
		}

		@Test
		void shouldCallService() {
			nachrichtenGrpcService.saveNewNachricht(newNachrichtRequest, newNachrichtStreamObserver);

			verify(nachrichtenService, atMostOnce()).save(any(NachrichtEvent.class));
		}

		@Test
		void shouldCreateNachricht() {
			nachrichtenGrpcService.saveNewNachricht(newNachrichtRequest, newNachrichtStreamObserver);

			verify(nachrichtenService, atMostOnce()).save(nachrichtArgumentCaptor.capture());

			var savedNachricht = nachrichtArgumentCaptor.getValue();

			assertThat(savedNachricht.getPostfachId()).isEqualTo(NachrichtEventTestFactory.POSTKORB_HANDLE);
			assertThat(savedNachricht.getNachrichtenListUrl()).isEqualTo(NachrichtEventTestFactory.URL);
		}

		@Test
		void shouldCallOnNext() {
			nachrichtenGrpcService.saveNewNachricht(newNachrichtRequest, newNachrichtStreamObserver);

			verify(newNachrichtStreamObserver, atMostOnce()).onNext(any(GrpcNewNachrichtReply.class));
		}

		@Test
		void shouldCallOnCompleted() {
			nachrichtenGrpcService.saveNewNachricht(newNachrichtRequest, newNachrichtStreamObserver);

			verify(newNachrichtStreamObserver, atMostOnce()).onCompleted();
		}

		@Test
		void shouldCallOnErrorDueToConstraintViolation() {
			doThrow(ConstraintViolationException.class).when(nachrichtenService).save(any(NachrichtEvent.class));

			nachrichtenGrpcService.saveNewNachricht(newNachrichtRequest, newNachrichtStreamObserver);

			verify(newNachrichtStreamObserver, atMostOnce()).onError(any());
		}

		@Test
		void shouldCallOnErrorDueToInvalidUrl() {
			doThrow(InvalidNachrichtenListUrlException.class).when(nachrichtenService).save(any(NachrichtEvent.class));

			nachrichtenGrpcService.saveNewNachricht(newNachrichtRequest, newNachrichtStreamObserver);

			verify(newNachrichtStreamObserver, atMostOnce()).onError(any());
		}
	}

	@Nested
	class TestSaveInvalidNewNachrichtEvent {
		@Mock
		private StreamObserver<GrpcNewNachrichtReply> newNachrichtStreamObserver;
		private final GrpcNachricht invalidNachricht = NachrichtEventTestFactory.createGrpcNachrichtBuilder().setNachrichtenListUrl("invalid")
		  .build();
		private final GrpcNewNachrichtRequest invalidNachrichtRequest = NachrichtEventTestFactory.createNewNachrichtRequestBuilder()
		  .setNachricht(invalidNachricht).build();

		@BeforeEach
		@SuppressWarnings("unchecked")
		void init() {
			when(nachrichtenMapper.fromGrpcNachricht(any())).thenReturn(
			  NachrichtEventTestFactory.createNachrichtEventBuilder().id(null).nachrichtenListUrl("invalid").build());
			var constraintViolation = mock(ConstraintViolation.class);

			when(validator.validate(any(NachrichtEvent.class))).thenReturn(Set.of(constraintViolation));
		}

		@Test
		void shouldNotCreateNachrichtBecauseInvalidNachrichtenListUrl() {
			nachrichtenGrpcService.saveNewNachricht(invalidNachrichtRequest, newNachrichtStreamObserver);

			verify(nachrichtenService, never()).save(any());
		}
	}

	@Nested
	class TestFinishNachrichtEvent {
		@Mock
		private StreamObserver<GrpcFinishedNachrichtReply> finishedNachrichtReplyStreamObserver;

		private final GrpcFinishedNachrichtRequest finishedNachrichtRequest = NachrichtEventTestFactory.createFinishNachrichtRequest();

		@BeforeEach
		void init() {
			when(nachrichtenMapper.fromGrpcNachricht(finishedNachrichtRequest.getNachricht())).thenReturn(
			  NachrichtEventTestFactory.createNachrichtEvent());
		}

		@Test
		void shouldCallService() {
			nachrichtenGrpcService.finishNachricht(finishedNachrichtRequest, finishedNachrichtReplyStreamObserver);

			verify(nachrichtenService, atMostOnce()).finish(any());
		}

		@Test
		void shouldDeleteNachricht() {
			nachrichtenGrpcService.finishNachricht(finishedNachrichtRequest, finishedNachrichtReplyStreamObserver);

			verify(nachrichtenService, atMostOnce()).finish(nachrichtArgumentCaptor.capture());

			var deletedNachricht = nachrichtArgumentCaptor.getValue();

			assertThat(deletedNachricht.getPostfachId()).isEqualTo(NachrichtEventTestFactory.POSTKORB_HANDLE);
			assertThat(deletedNachricht.getNachrichtenListUrl()).isEqualTo(NachrichtEventTestFactory.URL);
		}

		@Test
		void shouldCallOnNext() {
			nachrichtenGrpcService.finishNachricht(finishedNachrichtRequest, finishedNachrichtReplyStreamObserver);

			verify(finishedNachrichtReplyStreamObserver, atMostOnce()).onNext(any(GrpcFinishedNachrichtReply.class));
		}

		@Test
		void shouldCallOnCompleted() {
			nachrichtenGrpcService.finishNachricht(finishedNachrichtRequest, finishedNachrichtReplyStreamObserver);

			verify(finishedNachrichtReplyStreamObserver, atMostOnce()).onCompleted();
		}
	}
}