/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.info.nachricht;

import static de.ozgcloud.info.nachricht.NachrichtMapper.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.core.env.Environment;
import org.springframework.core.env.Profiles;

import de.ozgcloud.info.NachrichtEventTestFactory;
import de.ozgcloud.info.common.InvalidNachrichtenListUrlException;

@ExtendWith(MockitoExtension.class)
class NachrichtenMapperTest {
	@Spy
	@InjectMocks
	private NachrichtMapper nachrichtenMapper;

	@Mock
	private Environment environment;

	@Nested
	class TestFromGrpcNachricht {
		private final GrpcNachricht grpcNachricht = NachrichtEventTestFactory.createGrpcNachricht();

		@Test
		void shouldNotHaveId() {
			var nachrichtEvent = nachrichtenMapper.fromGrpcNachricht(grpcNachricht);

			assertThat(nachrichtEvent.getId()).isNull();
		}

		@Test
		void shouldHavePostfachId() {
			var nachrichtEvent = nachrichtenMapper.fromGrpcNachricht(grpcNachricht);

			assertThat(nachrichtEvent.getPostfachId()).isEqualTo(NachrichtEventTestFactory.POSTKORB_HANDLE);
		}

		@Test
		void shouldHaveNachrichtenListUrl() {
			var expectedNachrichtenListUrl = String.format(NACHRICHTEN_LIST_URL_FORMAT, NachrichtEventTestFactory.URL);
			var nachrichtEvent = nachrichtenMapper.fromGrpcNachricht(grpcNachricht);

			assertThat(nachrichtEvent.getNachrichtenListUrl()).isEqualTo(expectedNachrichtenListUrl);
		}
	}

	@Nested
	class TestGetNachrichtenListUrl {
		@BeforeEach
		void init() {
			when(environment.acceptsProfiles(Profiles.of(LOCAL_PROFILE))).thenReturn(false);
		}

		@Test
		void shouldPassOnUrl() {
			when(environment.acceptsProfiles(Profiles.of(LOCAL_PROFILE))).thenReturn(true);

			var grpcNachricht = NachrichtEventTestFactory.createGrpcNachricht();
			var result = nachrichtenMapper.getNachrichtenListUrl(grpcNachricht);

			assertThat(result).isEqualTo(NachrichtEventTestFactory.URL);
		}

		@DisplayName("should have valid format when")
		@ParameterizedTest(name = "url: {0}")
		@ValueSource(strings = { NachrichtEventTestFactory.URL, NachrichtEventTestFactory.URL_WITH_DNS, "test.example.com:20" })
		void shouldHaveValidFormat(String url) {
			var grpcNachricht = NachrichtEventTestFactory.createGrpcNachrichtBuilder().setNachrichtenListUrl(url).build();

			assertThatNoException().isThrownBy(() -> nachrichtenMapper.getNachrichtenListUrl(grpcNachricht));
		}

		@ParameterizedTest
		@ValueSource(strings = { "invalidUrl", "%test%:2345", "localhost:234567", "dns://host:9090" })
		void shouldHaveInvalidFormat(String url) {
			var grpcNachricht = NachrichtEventTestFactory.createGrpcNachrichtBuilder().setNachrichtenListUrl(url).build();

			assertThatExceptionOfType(InvalidNachrichtenListUrlException.class).isThrownBy(
			  () -> nachrichtenMapper.getNachrichtenListUrl(grpcNachricht));
		}

		@Test
		void shouldAddDnsPrefix() {
			var result = nachrichtenMapper.getNachrichtenListUrl(NachrichtEventTestFactory.createGrpcNachricht());

			assertThat(result).isEqualTo(NachrichtEventTestFactory.URL_WITH_DNS);
		}

		@Test
		void shouldNotAddDnsPrefix() {
			var grpcNachricht = NachrichtEventTestFactory.createGrpcNachrichtBuilder().setNachrichtenListUrl(NachrichtEventTestFactory.URL_WITH_DNS)
			  .build();

			var result = nachrichtenMapper.getNachrichtenListUrl(grpcNachricht);

			assertThat(result).isEqualTo(NachrichtEventTestFactory.URL_WITH_DNS);
		}
	}
}